package MT::Plugin::SidebarImage;
use strict;
use MT;
use MT::Plugin;
use MT::Asset;
use MT::FileInfo;
use MT::ObjectAsset;
use File::Spec;
use File::Basename;
use Image::Size;
use MT::Util qw( encode_url perl_sha1_digest_hex );

our $VERSION = '0.3';

use base qw( MT::Plugin );

@MT::Plugin::SidebarImage::ISA = qw(MT::Plugin);

my $plugin = new MT::Plugin::SidebarImage({
    name        => 'SidebarImage',
    key         => 'sidebarimage',
    description => '<MT_TRANS phrase=\'_PLUGIN_DESCRIPTION\'>',
    author_name => 'Junnama Noda',
    author_link => 'http://junnama.alfasado.net/online/',
    settings => new MT::PluginSettings([
        ['directory', { Default => 'files' }],
    ]),
    version     => $VERSION,
    l10n_class  => 'SidebarImage::L10N',
    blog_config_template => 'sidebarimage_cfg.tmpl',
});

MT->add_plugin($plugin);

sub init_registry {
    my $plugin = shift;
    $plugin->registry({
        applications => {
            cms => {
                methods => {
                    sidebarimage =>
                        'MT::Plugin::SidebarImage::_sidebarimage',
                },
            },
        },
        callbacks => {
            'MT::App::CMS::template_source.edit_entry'
                => {
                    handler => \&_sidebarimage_source,
                    priority => 9,
                },
            'MT::App::CMS::template_source.edit_asset'
                => \&_edit_asset_dialog,
            'MT::App::CMS::template_output.edit_asset'
                => \&_edit_asset_output,
            'cms_post_save.entry'
                => \&_parse_asset,
            'cms_post_save.page'
                => \&_parse_asset,
        },
   });
}

our %mime_type = (
    'css'   =>  'text/css',
    'html'  =>  'text/html',
    'htm'   =>  'text/html',
    'txt'   =>  'text/plain',
    'rtx'   =>  'text/richtext',
    'tsv'   =>  'text/tab-separated-values',
    'hdml'  =>  'text/x-hdml;charset=Shift_JIS',    
    'xml'   =>  'application/rss+xml',
    'rdf'   =>  'application/rss+xml',
    'xsl'   =>  'text/xsl',
    'mpeg'  =>  'video/mpeg',
    'mpg'   =>  'video/mpeg',
    'mpe'   =>  'video/mpeg',
    'qt'    =>  'video/quicktime',
    'avi'   =>  'video/x-msvideo',
    'movie' =>  'video/x-sgi-movie',
    'qt'    =>  'video/quicktime',
    'ice'   =>  'x-conference/x-cooltalk',
    'svr'   =>  'x-world/x-svr',
    'vrml'  =>  'x-world/x-vrml',
    'wrl'   =>  'x-world/x-vrml',
    'vrt'   =>  'x-world/x-vrt',    
    'spl'   =>  'application/futuresplash',
    'hqx'   =>  'application/mac-binhex40',
    'doc'   =>  'application/msword',
    'pdf'   =>  'application/pdf',
    'ai'    =>  'application/postscript',
    'eps'   =>  'application/postscript',
    'ps'    =>  'application/postscript',
    'ppt'   =>  'application/powerpoint',
    'rtf'   =>  'application/rtf',
    'dcr'   =>  'application/x-director',
    'dir'   =>  'application/x-director',
    'dxr'   =>  'application/x-director',
    'js'    =>  'application/x-javascript',
    'dvi'   =>  'application/x-dvi',
    'gtar'  =>  'application/x-gtar',
    'gzip'  =>  'application/x-gzip',
    'latex' =>  'application/x-latex',
    'lzh'   =>  'application/x-lha',
    'swf'   =>  'application/x-shockwave-flash',
    'sit'   =>  'application/x-stuffit',
    'tar'   =>  'application/x-tar',
    'tcl'   =>  'application/x-tcl',
    'tex'   =>  'application/x-texinfo',
    'texinfo'=> 'application/x-texinfo',
    'texi'  =>  'application/x-texi',
    'src'   =>  'application/x-wais-source',
    'zip'   =>  'application/zip',
    'au'    =>  'audio/basic',
    'snd'   =>  'audio/basic',
    'midi'  =>  'audio/midi',
    'mid'   =>  'audio/midi',
    'kar'   =>  'audio/midi',
    'mpga'  =>  'audio/mpeg',
    'mp2'   =>  'audio/mpeg',
    'mp3'   =>  'audio/mpeg',
    'ra'    =>  'audio/x-pn-realaudio',
    'ram'   =>  'audio/x-pn-realaudio',
    'rm'    =>  'audio/x-pn-realaudio',
    'rpm'   =>  'x-pn-realaudio-plugin',
    'wav'   =>  'audio/x-wav',
    'bmp'   =>  'image/bmp',
    'gif'   =>  'image/gif',
    'jpeg'  =>  'image/jpeg',
    'jpg'   =>  'image/jpeg',
    'jpe'   =>  'image/jpeg',
    'png'   =>  'image/png',
    'tiff'  =>  'image/tiff',
    'tif'   =>  'image/tiff',
    'pnm'   =>  'image/x-portable-anymap',
    'ras'   =>  'image/x-cmu-raster',
    'pnm'   =>  'image/x-portable-anymap',
    'pbm'   =>  'image/x-portable-bitmap',
    'pgm'   =>  'image/x-portable-graymap',
    'ppm'   =>  'image/x-portable-pixmap',
    'rgb'   =>  'image/x-rgb',
    'xbm'   =>  'image/x-xbitmap',
    'xpm'   =>  'image/x-pixmap',
    'xwd'   =>  'image/x-xwindowdump',
);

sub _parse_asset {
    my ( $eh, $app, $obj, $original ) = @_;
    my $blog = $app->blog;
    my $site_path = _site_path( $blog );
    my $site_url  = _site_url ( $blog );
    my $abs_path  = $site_url;
    $abs_path =~ s/https*:\/\/.*?(\/.*$)/$1/;
    $abs_path = quotemeta( $abs_path );
    my $doc_root = $site_path;
    $doc_root =~ s/$abs_path$//;
    if ( $doc_root =~ /(.*)\/$/ ) {
        $doc_root = $1;
    }
    if ( $^O eq 'MSWin32' ) {
        if ( $doc_root =~ /(.*)\\$/ ) {
            $doc_root = $1;
        }
    }
    my $file = $obj->archive_file();
    $file = File::Spec->catfile ( $site_path, $file );
    my $dir = File::Basename::dirname( $file );
    my $match = '<[^>]+\s(src|href|action)\s*=\s*\"';
    my $text = $obj->text;
    $text =~ s/($match)(.{1,}?)(")/$1.&_check_asset(
             $app, $blog, $obj, $3, $dir, $site_path, $site_url, $doc_root ).$4/esg;
    my $text_more = $obj->text_more;
    $text_more =~ s/($match)(.{1,}?)(")/$1.&_check_asset(
             $app, $blog, $obj, $3, $dir, $site_path, $site_url, $doc_root ).$4/esg;
}

sub _check_asset {
    my ( $app, $blog, $obj, $src, $file, $site_path, $site_url, $doc_root ) = @_;
    my $path = $src;
    my $full_path;
    if ( $path =~ m!^\.\./! ) {
        $path = File::Spec->rel2abs( $path, $file );
        my $match;
        while (! $match ) {
            my $orginal = $path;
            $path =~ s!/[^/]*?/\.\./!/!sg;
            if ( $orginal eq $path ) {
                $match = 1;
            }
        }
        $full_path = $path;
        $path =~ s/$site_path/%r/;
    } elsif ( $path =~ /^$site_url/ ) {
        $path =~ s/$site_url/%r/;
        $full_path = $path;
        $full_path =~ s/%r/$site_path/;
    } elsif ( $path =~ m!^/(.*)! ) {
        $path = File::Spec->catfile ( $doc_root, $1 );
        $full_path = $path;
        $path =~ s/$site_path/%r/;
    }
    if ( $full_path ) {
        if ( -f $full_path ) {
            my $fileinfo = MT::FileInfo->load( { file_path => $full_path } );
            unless ( defined $fileinfo ) {

                my $asset = MT::Asset->load( { class => '*',
                                               url   => $path } );
                if ( defined $asset ) {
                    # require MT::ObjectAsset;
                    my $objectasset = MT::ObjectAsset->get_by_key( { asset_id => $asset->id,
                                                                     object_id => $obj->id,
                                                                     object_ds => 'entry',
                                                                     blog_id => $blog->id
                                                                    } );
                    unless ( $objectasset->id ) {
                        $objectasset->save
                                or return $app->trans_error('Error saving objectasset: [_1]', $objectasset->errstr);
                    }
                }
            }
        }
    }
    return $src;
}

sub _sidebarimage {
    my $app = shift;
    $app->validate_magic() or return;
    return $app->errtrans("Permission denied.")
        unless $app->can_do( 'save_asset' );
    my $q = $app->param;
    my $blog_id = $q->param( 'blog_id' );
    return $app->errtrans("Permission denied.")
        unless $blog_id;
    my $blog = $app->blog;
    my $site_path = _site_path( $blog );
    my $site_url  = _site_url ( $blog );
    my $offset = $q->param( 'offset' );
    my $type = $q->param( 'type' );
    if ( $type eq 'upload' ) {
        my $get_from = 'blog:'. $blog_id;
        my $directory = $plugin->get_config_value( 'directory', $get_from );
        for my $key ( $q->param ) {
            if ( $key =~ /^sidebar_image_([0-9]{1,})$/ ) {
                my $num = $1;
                my $fmgr = $blog->file_mgr;
                my $file = $q->upload( 'sidebar_image_' . $num );
                if ( $file ) {
                    my $label = $q->param( 'sidebar_name_' . $num );
                    my $out = File::Spec->catfile ( $site_path, $directory, _file_name($file) );
                    my $dir = File::Basename::dirname( $out );
                    $dir =~ s!/$!! unless $dir eq '/';
                    unless ( $fmgr->exists( $dir ) ) {
                        $fmgr->mkpath( $dir );
                    }
                    my $temp = "$out.new";
                    open ( OUT, ">$out" ) or die "Can't open $out!";
                    binmode ( OUT );
                    while( read ( $file, my $buffer, 1024 ) ) {
                        print OUT $buffer;
                    }
                    close (OUT);
                    $fmgr->rename( $temp, $out );
                    $out =~ s/$site_path/%r/;
                    my %asset_elements = (
                        'parent' => undef,
                        'file' => $out,
                        'blog_id' => $blog->id,
                        'label' => $label,
                        'description' => '',
                        'author_id' => $app->user->id,
                    );
                    my $asset = &_save_asset( $app, %asset_elements, 1 );
                }
            }
        }
    }
    if ( $offset !~ /[0-9]+/ ) {
        $offset = 0;
    }
    my $limit = 15;
    my $count = MT::Asset::Image->count ( { blog_id => $blog_id } );
    my $iter = MT::Asset::Image->load_iter ( { blog_id => $blog_id },
                                             { offset  => $offset,
                                               limit   => $limit,
                                               sort    => 'modified_on',
                                               direction => 'descend'
                                             } );
    my %param;
    my @asset_loop; my $odd = 1; my $i = 1;
    $app->{plugin_template_path} = File::Spec->catdir($plugin->path,'tmpl');
    while ( my $asset = $iter->() ) {
        my $asset_path = $asset->file_path;
        $asset_path =~ s/^%r/$site_path/;
        if ( -f $asset_path ) {
            my $asset_url = $asset->url;
            my $asset_name = $asset->label;
            $asset_name = $asset->file_name unless ( $asset_name );
            my %arg;
            $arg{Width} = 75;
            my ( $thumbnail_url, $thumbnail_w, $thumbnail_h ) = $asset->thumbnail_url( %arg );
            push @asset_loop,
                {
                    asset_name    => $asset_name,
                    asset_label   => $asset->label,
                    asset_width   => $asset->image_width,
                    asset_height  => $asset->image_height,
                    asset_url     => $asset->url,
                    thumbnail_url => $thumbnail_url,
                    asset_num     => $i,
                    asset_id      => $asset->id,
                    odd => $odd,
                };
            if ( $odd == 1 ) { $odd = 0 } else { $odd = 1 };
            $i++;
        }
    }
    my $tmpl = 'sidebar_image.tmpl';
    $param{'offset'} = $offset;
    if ( $offset ) {
        my $prev_offset = $offset - $limit;
        if ( $prev_offset > -1 ) {
            $param{'prev_offset'} = 1;
            $param{'prev_offset_val'} = $prev_offset;
        }
    }
    $param{'list_total'} = $count;
    $param{'list_start'} = $offset + 1;
    my $list_end = $offset + $i - 1;
    $param{'list_end'} = $list_end;
    if ( $list_end < $count ) {
        $param{'next_offset'} = 1;
        $param{'next_offset_val'} = $offset + $limit;
        $param{'next_max'} = $count - $limit;
    }
    $param{'asset_loop'} = \@asset_loop;
    return $app->build_page( $tmpl, \%param );
}

sub _sidebarimage_source {
    my ( $cb, $app, $tmpl ) = @_;
    my $static_uri = $app->static_path;
    my $src = $app->base . $static_uri . 'images/indicator.gif';
    my $append = <<MTML;
<mt:setvarblock name="related_content" prepend="1">
<div class="widget pkg customfields-reorder-widget">
    <div class="widget-inner inner">
        <div class="widget-header ">
            <div class="widget-header-inner pkg">
                <h3 class="widget-label"><a id="sidebar-image-widget" href="javascript:void(0)" onclick="toggle_canvas()"><span><__trans phrase="Image"></span></a></h3>
            </div>
        </div>
    </div>
    <div class="widget-content" id="create_sidebar_image">
        <div class="widget-content-inner">
            <p class="create-link" id="create-link"><a class="icon-left icon-create"
            href="javascript:void(0)"
            onclick="toggle_append_image()"
            ><__trans phrase="Create"></a></p>
        </div>
    </div>
</div>

<div style="margin-bottom:12px; display:none;" id="image_canvas">
    <iframe id="canvas-iframe" height="300" frameborder="0"></iframe>
</div>

<script type="text/javascript">
    var isIE = (document.documentElement.getAttribute("style") == document.documentElement.style);
    var loaded;
    var flds = 1;
    function toggle_canvas () {
        var canvas = document.getElementById('image_canvas');
        if ( canvas.style.display == 'none' ) {
            if ( ! loaded ) {
                try {
                    appendHTML('canvas-iframe', '$src');
                } catch (e) {
                }
                var rnd = Math.random();
                document.getElementById('canvas-iframe').src = '<mt:var name="script_url">?__mode=sidebarimage&amp;blog_id=<mt:var name="blog_id">&amp;offset=0&amp;key=' + rnd;
                loaded = 1;
            }
            canvas.style.display='block';
            document.getElementById('create_sidebar_image').style.display = 'block';
            document.getElementById('sidebar-image-widget').style.backgroundImage = 'url(<mt:var name="static_uri">images/spinner-bottom.gif)';
        } else {
            canvas.style.display='none';
            document.getElementById('create_sidebar_image').style.display = 'none';
            document.getElementById('sidebar-image-widget').style.backgroundImage = 'url(<mt:var name="static_uri">images/spinner-right.gif)';
        }
    }
    function iframeDoc( id ) {
        if ( isIE ) {
            return frames[id].document;
        } else {
            return document.getElementById(id).contentDocument;
        }
    }
    function appendHTML( iframeid, src ) {
        var doc = iframeDoc( iframeid );
        var container = doc.createElement("div");
        container.style.height = '165px';
        container.style.backgroundImage = 'url("' + src + '")';
        container.style.backgroundRepeat = 'no-repeat';
        container.style.backgroundPosition = 'center bottom';
        doc.body.appendChild(container);
    }
    function toggle_append_image () {
        var doc = iframeDoc( 'canvas-iframe' );
        var ele = doc.getElementById('append_image');
        ele.style.display = 'block';
        ele = doc.getElementById('sidebar_image_block');
        var block = doc.createElement('div');
        block.setAttribute( 'id', 'sidebar_block_' + flds );
        var label = doc.createElement('label');
        var label_val = doc.createTextNode('<__trans phrase="Name">');
        label.appendChild( label_val );
        label.setAttribute( 'for', 'sidebar_name_' + flds );
        block.appendChild( label );
        var wrapper = doc.createElement('div');
        wrapper.className = 'textarea-wrapper';
        var field = doc.createElement('input');
        field.className = 'sidebar_name';
        field.setAttribute( 'id', 'sidebar_name_' + flds );
        field.setAttribute( 'name', 'sidebar_name_' + flds );
        field.setAttribute( 'type', 'text' );
        wrapper.appendChild( field );
        block.appendChild( wrapper );
        para = doc.createElement('p');
        field = doc.createElement('input');
        field.className = 'sidebar_image';
        field.setAttribute( 'id', 'sidebar_image_' + flds );
        field.setAttribute( 'name', 'sidebar_image_' + flds );
        field.setAttribute( 'type', 'file' );
        para.appendChild( field );
        block.appendChild( para );
        ele.appendChild( block );
        flds++;
    }
</script>
</mt:setvarblock>

<mt:setvarblock name="html_head" append="1">
<style type="text/css">
#create_sidebar_image {
    display: none;
}
#sidebar-image-widget {
    background: url(<mt:var name="static_uri">images/spinner-right.gif) no-repeat left center;
    padding-left: 11px;
}
</style>
</mt:setvarblock>
<mt:include name="include/header.tmpl" id="header_include">
MTML
    my $include = '<mt:include name="include/header.tmpl" id="header_include">';
    my $old = quotemeta($include);
    $$tmpl =~ s/$old/$append/;
}

sub _edit_asset_dialog {
    my ($cb, $app, $tmpl) = @_;
    my $q = $app->param;
    my $flag = $q->param('sidebar');
    if ( $flag ) {
        my $include = '<mt:include name="include/header.tmpl">';
        my $dialog = '<mt:include name="dialog/header.tmpl">';
        $dialog .= '<div style="width:50%">';
        $$tmpl =~ s/$include/$dialog/;
        $dialog = '<mt:include name="dialog/footer.tmpl">';
        $include = '<mt:include name="include/footer.tmpl">';
        $$tmpl =~ s/$include/$dialog/;
        my $button = '<__trans phrase="Save Changes"></button>';
        my $cancel =<<BUTTON;
        <__trans phrase="Save Changes"></button>
        <button
            onclick="closeDialog(); return false"
            type="submit"
            accesskey="x"
            title="<__trans phrase="Cancel"> (x)"
            ><__trans phrase="Cancel"></button>
BUTTON
        $$tmpl =~ s/$button/$cancel/;
    }
}

sub _edit_asset_output {
    my ($cb, $app, $tmpl) = @_;
    my $q = $app->param;
    my $icon = $app->static_path . 'images/asset/file.gif';
    my $icon_v = $app->static_path . 'images/asset/video.gif';
    my $icon_a = $app->static_path . 'images/asset/audio.gif';
    my $flag = $q->param('sidebar');
    if ( $flag ) {
        my $head = '</head>';
        my $css .=<<"CSS";
<style type="text/css">
.asset-thumb-metadata {background-color:white;text-align:center}
.asset-type-image {background-color:transparent !important}
.asset-type-file {background-color:transparent !important}
.asset-type-video {background-color:transparent !important}
.asset-type-audio {background-color:transparent !important}
.asset-type-image .asset-thumb-inner{ background-color:black;text-align:center }
.asset-type-file .asset-thumb-inner { height:250px;text-indent:-10000px; background-color:#EEEEEE;
                                      background-image:url($icon);
                                      background-repeat:no-repeat;
                                      background-position:120px 120px;
                                      border:1px solid gray; }
.asset-type-video .asset-thumb-inner{ height:250px;text-indent:-10000px; background-color:#EEEEEE;
                                      background-image:url($icon_v);
                                      background-repeat:no-repeat;
                                      background-position:120px 120px;
                                      border:1px solid gray; }
.asset-type-audio .asset-thumb-inner{ height:250px;text-indent:-10000px; background-color:#EEEEEE;
                                      background-image:url($icon_a);
                                      background-repeat:no-repeat;
                                      background-position:120px 120px;
                                      border:1px solid gray; }
</style>
</head>
CSS
        $$tmpl =~ s/$head/$css/;
        my $return_args = '<input type="hidden" name="return_args" value="';
        $$tmpl =~ s/($return_args.*?)"/$1&amp;dialog=1"/;
    }
}

sub _save_asset {
    my $app = shift;
    my %asset_elements = @_;
    my $callback = shift;
    require MT::Asset;
    my $file_path = $asset_elements{'file'};
    if ($^O eq 'MSWin32') {
        $file_path =~ s/\//\\/;
    }
    my $parent = $asset_elements{'parent'};
    if ( ($^O eq 'MSWin32') && ($parent) ) {
        $parent =~ s/\//\\/;
    }
    my $author_id = $asset_elements{'author_id'};
    my $blog_id = $asset_elements{'blog_id'};
    my $label = $asset_elements{'label'};
    my $description = $asset_elements{'description'};
    my $basename = File::Basename::basename($file_path);
    my @suffix = split(/\./, $file_path);
    my $ext = pop(@suffix);
    $ext = lc($ext);
    my $mime_type = $mime_type{$ext};
    my $asset_class = 'file';
    if ( ($ext =~ /^jpe?g$/i) || ($ext =~ /^gif$/i) || ($ext =~ /^png$/i) ) {
        $asset_class = 'image';
    } elsif (($ext =~ /^mov$/i) || ($ext =~ /^avi$/i) || ($ext =~ /^3gp$/i)
                || ($ext =~ /^asf$/i) || ($ext =~ /^mp4$/i) || ($ext =~ /^qt$/i)
                || ($ext =~ /^wmv$/i) || ($ext =~ /^asx$/i) || ($ext =~ /^mpg$/i)
            ) {
        $asset_class = 'video';
    } elsif ( ($ext =~ /^mp3$/i) || ($ext =~ /^ogg$/i) || ($ext =~ /^aiff?$/i)
                || ($ext =~ /^wav$/i) || ($ext =~ /^wma$/i) || ($ext =~ /^aac$/i)
            ) {
        $asset_class = 'audio';
    }
    unless ($author_id) {
        my $author = MT::Author->load();
        $author_id = $author->id;
    }
    my $separator = '\/';
    if ($^O eq 'MSWin32') {
        $separator = '\\';
    }
    my $blog; my $asset_parant_id;
    unless ($blog_id) {
        my @blogs = MT::Blog->load();
        for my $item (@blogs) {
            my $check_path = $item->site_path;
            $check_path = quotemeta($check_path);
            if ($file_path =~ /^$check_path/) {
                $blog = $item;
                last;
            }
        }
    } else {
        $blog = MT::Blog->load( { id => $blog_id } );
    }
    $blog_id = $blog->id;
    my $site_path = $blog->site_path;
    if ($site_path =~ /(^.*)$separator$/) {
        $site_path = $1;
    }
    $site_path = quotemeta($site_path);
    my $asset_path = $file_path;
    $asset_path =~ s/$site_path/%r/;
    my $asset_url = $asset_path;
    if ($^O eq 'MSWin32') {
        $asset_url =~ s/\\/\//g;
    }
    if ( $parent ) {
        my $parent_basename = File::Basename::basename($parent);
        $parent =~ s/$site_path/%r/;
        my $parent_asset_pkg = MT::Asset->handler_for_file($parent_basename);
        if ($asset_class eq 'image') {
            $parent_asset_pkg->isa('MT::Asset::Image');
        } elsif ($asset_class eq 'video') {
            $parent_asset_pkg->isa('MT::Asset::Video');
        } elsif ($asset_class eq 'audio') {
            $parent_asset_pkg->isa('MT::Asset::Audio');
        }
        my $asset_parant = $parent_asset_pkg->load( { file_path => $parent } );
        if (defined $asset_parant) {
            if ($asset_parant->blog_id != $blog_id) {
                $blog_id = $asset_parant->blog_id;
                $blog = MT::Blog->load( { id => $blog_id } );
                $site_path = $blog->site_path;
                if ($site_path =~ /(^.*)$separator$/) {
                    $site_path = $1;
                }
                $site_path = quotemeta($site_path);
                my $asset_path = $file_path;
                $asset_path =~ s/$site_path/%r/;
            }
            $asset_parant_id = $asset_parant->id;
            unless ($label) {
                $label = $asset_parant->label;
            }
            unless ($description) {
                $description = $asset_parant->description;
            }
        }
    }
    my $asset_pkg = MT::Asset->handler_for_file($basename);
    my $asset; my $is_image;
    if ($asset_class eq 'image') {
        $asset_pkg->isa('MT::Asset::Image');
        $is_image = 1;
    } elsif ($asset_class eq 'video') {
        $asset_pkg->isa('MT::Asset::Video');
    } elsif ($asset_class eq 'audio') {
        $asset_pkg->isa('MT::Asset::Audio');
    }
    if (!( $asset = $asset_pkg->load( { file_path => $asset_path, blog_id => $blog_id } ))) {
        $asset = $asset_pkg->new();
    }
    unless (defined $asset) {
        $asset = MT::Asset->load( { class     => '*',
                                    file_path => $asset_path } );
        if (defined $asset) {
        } else {
            $asset = MT::Asset->new;
        }
    }
    $asset->blog_id($blog_id);
    $asset->file_path($asset_path);
    $asset->url($asset_url);
    $asset->file_name($basename);
    $asset->mime_type($mime_type);
    $asset->file_ext($ext);
    $asset->class($asset_class);
    $asset->created_by($author_id);
    $asset->modified_by($author_id);
    if ($asset_parant_id) {
        $asset->parent($asset_parant_id);
    }
    my ($w,$h);
    if ($asset_class eq 'image') {
        ($w,$h) = Image::Size::imgsize($file_path);
        $asset->image_width($w);
        $asset->image_height($h);
    }
    if ($label) {
        $asset->label($label);
    }
    if ($description) {
        $asset->description($description);
    }
    $asset->save
        or return $plugin->translate('Error saving asset: [_1]', $asset->errstr);
    if ( $callback ) {
        my $site_url  = _site_url( $blog );
        my @fdatas = stat($file_path);
        my $bytes = $fdatas[7];
        my $res = _upload_callback( $app, $file_path, $asset_url, $bytes,
                                    $asset, $blog, $w, $h, $asset_class, $is_image );
    }
    return $asset;
}

sub _upload_callback {
    my ($app, $file, $url, $bytes, $asset, $blog, $h, $w, $type, $is_image) = @_; 
    if ($is_image) {
        $app->run_callbacks(
            'cms_upload_file.' . $asset->class,
            File  => $file,
            file  => $file,
            Url   => $url,
            url   => $url,
            Size  => $bytes,
            size  => $bytes,
            Asset => $asset,
            asset => $asset,
            Type  => 'image',
            type  => 'image',
            Blog  => $blog,
            blog  => $blog
        );
        $app->run_callbacks(
            'cms_upload_image',
            File       => $file,
            file       => $file,
            Url        => $url,
            url        => $url,
            Size       => $bytes,
            size       => $bytes,
            Asset      => $asset,
            asset      => $asset,
            Height     => $h,
            height     => $h,
            Width      => $w,
            width      => $w,
            Type       => 'image',
            type       => 'image',
            ImageType  => $type,
            image_type => $type,
            Blog       => $blog,
            blog       => $blog
        );
    } else {
        $app->run_callbacks(
            'cms_upload_file.' . $asset->class,
            File  => $file,
            file  => $file,
            Url   => $url,
            url   => $url,
            Size  => $bytes,
            size  => $bytes,
            Asset => $asset,
            asset => $asset,
            Type  => 'file',
            type  => 'file',
            Blog  => $blog,
            blog  => $blog
        );
    }
    return 1;
}

sub _site_path {
    my $blog = shift;
    my $site_path = $blog->site_path;
    if ( $site_path =~ /(.*)\/$/ ) {
        $site_path = $1;
    }
    if ( $^O eq 'MSWin32' ) {
        if ( $site_path =~ /(.*)\\$/ ) {
            $site_path = $1;
        }
    }
    return $site_path;
}

sub _site_url {
    my $blog = shift;
    my $site_url = $blog->site_url;
    if ( $site_url =~ /(.*)\/$/ ) {
        $site_url = $1;
    }
    return $site_url;
}

sub _file_name {
    my $file = shift;
    $file =~ s/\\/\//g;
    $file =~ s/://g;
    if ( $file =~ /\// ) {
        my @pathes = split( /\//, $file );
        $file = pop(@pathes);
    }
    my $_ctext = encode_url( $file );
    if ($_ctext ne $file) {
        my @suffix = split( /\./, $file );
        my $ext = pop(@suffix);
        my $ext_len = length($ext) + 1;
        if (eval { require Digest::MD5; 1; }) {
            $file = Digest::MD5::md5_hex($file);
        } else {
            $file = perl_sha1_digest_hex($file);
        }
        $file = substr ( $file, 0, 255 - $ext_len );
        $file .= '.' . $ext;
    }
    return $file;
}

1;